@@ -4,21 +4,19 @@ Notes on pg Provider
 This provider implements basic funcionalities from asyncpg
 (cursors, transactions, copy from and to files, pools, native data types, etc).
 """
-import os
 import asyncio
+import os
+import ssl
 import time
 import uuid
-from typing import (
-    Optional,
-    Any,
-    Union
-)
-from collections.abc import Iterable, Callable
-import uvloop
+from collections.abc import Callable, Iterable
+from typing import Any, Optional, Union
+
 import asyncpg
-from asyncpg.pgproto import pgproto
+import uvloop
 from asyncpg.exceptions import (
     ConnectionDoesNotExistError,
+    DuplicateTableError,
     FatalPostgresError,
     InterfaceError,
     InterfaceWarning,
@@ -29,32 +27,26 @@ from asyncpg.exceptions import (
     TooManyConnectionsError,
     UndefinedColumnError,
     UndefinedTableError,
-    DuplicateTableError,
-    UniqueViolationError
+    UniqueViolationError,
 )
+from asyncpg.pgproto import pgproto
+
 from asyncdb.exceptions import (
-    DriverError,
     ConnectionTimeout,
+    DriverError,
     EmptyStatement,
     ProviderError,
     StatementError,
     TooManyConnections,
-    UninitializedError
+    UninitializedError,
 )
-from asyncdb.interfaces import (
-    DBCursorBackend
-)
-from asyncdb.utils.encoders import (
-    DefaultEncoder
-)
-from asyncdb.utils.types import Entity
+from asyncdb.interfaces import DBCursorBackend, ModelBackend
 from asyncdb.models import Model
-from asyncdb.interfaces import ModelBackend
-from .abstract import (
-    BasePool
-)
-from .sql import SQLDriver, SQLCursor
+from asyncdb.utils.encoders import DefaultEncoder
+from asyncdb.utils.types import Entity
 
+from .abstract import BasePool
+from .sql import SQLCursor, SQLDriver
 
 max_cached_statement_lifetime = 600
 max_cacheable_statement_size = 1024 * 15
@@ -71,7 +63,7 @@ class pgPool(BasePool):
     _setup_func: Optional[Callable] = None
     _init_func: Optional[Callable] = None
 
-    def __init__(self, dsn="", loop=None, params: Optional[dict] = None, **kwargs):
+    def __init__(self, dsn: str = None, loop: asyncio.AbstractEventLoop = None, params: Optional[dict] = None, **kwargs):
         self._test_query = 'SELECT 1'
         self.application_name = os.getenv('APP_NAME', "NAV")
         self._max_clients = 300
@@ -81,7 +73,6 @@ class pgPool(BasePool):
         super(pgPool, self).__init__(
             dsn=dsn, loop=loop, params=params, **kwargs
         )
-        print('DSN is: ', self._dsn)
         if "server_settings" in kwargs:
             self._server_settings = kwargs["server_settings"]
         if "application_name" in self._server_settings:
@@ -94,6 +85,48 @@ class pgPool(BasePool):
             self._numeric_as_float = kwargs['numeric_as_float']
         # set the JSON encoder:
         self._encoder = DefaultEncoder()
+        ### SSL Support:
+        self.ssl: bool = False
+        if params and 'ssl' in params:
+            ssloptions = params['ssl']
+        elif 'ssl' in kwargs:
+            ssloptions = kwargs['ssl']
+        else:
+            ssloptions = None
+        if ssloptions:
+            self.ssl: bool = True
+            try:
+                check_hostname = ssloptions['check_hostname']
+            except KeyError:
+                check_hostname = False
+            ### certificate Support:
+            try:
+                ca_file = ssloptions['cafile']
+            except KeyError:
+                ca_file = None
+            args = {
+                "cafile": ca_file
+            }
+            self.sslctx = ssl.create_default_context(
+                ssl.Purpose.SERVER_AUTH,
+                **args
+            )
+            # Certificate Chain:
+            try:
+                certs = {
+                    "certfile": ssloptions['certfile'],
+                    "keyfile": ssloptions['keyfile']
+                }
+            except KeyError:
+                certs = {
+                    "certfile": None,
+                    "keyfile": None
+                }
+            if certs['certfile']:
+                self.sslctx.load_cert_chain(
+                    **certs
+                )
+            self.sslctx.check_hostname = check_hostname
 
     async def setup_connection(self, connection):
         if self._setup_func:
@@ -172,9 +205,6 @@ class pgPool(BasePool):
         self._logger.debug(
             f"AsyncPg (Pool): Connecting to {self._dsn}"
         )
-        print(
-            f"AsyncPg (Pool): Connecting to {self._dsn}"
-        )
         try:
             # TODO: pass a setup class for set_builtin_type_codec and a setup for add listener
             server_settings = {
@@ -185,6 +215,12 @@ class pgPool(BasePool):
                 "jit": "off"
             }
             server_settings = {**server_settings, **self._server_settings}
+            if self.ssl:
+                _ssl = {
+                    "ssl": self.sslctx
+                }
+            else:
+                _ssl = {}
             self._pool = await asyncpg.create_pool(
                 dsn=self._dsn,
                 max_queries=self._max_queries,
@@ -197,7 +233,8 @@ class pgPool(BasePool):
                 setup=self.setup_connection,
                 loop=self._loop,
                 server_settings=server_settings,
-                connection_class=NAVConnection
+                connection_class=NAVConnection,
+                **_ssl
             )
             # is connected
             if self._pool:
@@ -454,6 +491,48 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
             self._numeric_as_float = False
         # set the JSON encoder:
         self._encoder = DefaultEncoder()
+        ### SSL Support:
+        self.ssl: bool = False
+        if params and 'ssl' in params:
+            ssloptions = params['ssl']
+        elif 'ssl' in kwargs:
+            ssloptions = kwargs['ssl']
+        else:
+            ssloptions = None
+        if ssloptions:
+            self.ssl: bool = True
+            try:
+                check_hostname = ssloptions['check_hostname']
+            except KeyError:
+                check_hostname = False
+            ### certificate Support:
+            try:
+                ca_file = ssloptions['cafile']
+            except KeyError:
+                ca_file = None
+            args = {
+                "cafile": ca_file
+            }
+            self.sslctx = ssl.create_default_context(
+                ssl.Purpose.SERVER_AUTH,
+                **args
+            )
+            # Certificate Chain:
+            try:
+                certs = {
+                    "certfile": ssloptions['certfile'],
+                    "keyfile": ssloptions['keyfile']
+                }
+            except KeyError:
+                certs = {
+                    "certfile": None,
+                    "keyfile": None
+                }
+            if certs['certfile']:
+                self.sslctx.load_cert_chain(
+                    **certs
+                )
+            self.sslctx.check_hostname = check_hostname
 
     async def close(self, timeout=5):
         """
@@ -530,7 +609,12 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
             "jit": "off"
         }
         server_settings = {**server_settings, **self._server_settings}
-
+        if self.ssl:
+            _ssl = {
+                "ssl": self.sslctx
+            }
+        else:
+            _ssl = {}
         try:
             if self._pool and not self._connection:
                 self._connection = await self._pool.pool().acquire()
@@ -542,7 +626,9 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
                     max_cached_statement_lifetime=max_cached_statement_lifetime,
                     max_cacheable_statement_size=max_cacheable_statement_size,
                     server_settings=server_settings,
-                    connection_class=NAVConnection
+                    connection_class=NAVConnection,
+                    loop=self._loop,
+                    **_ssl
                 )
                 await self._connection.set_type_codec(
                     "json",
@@ -723,6 +809,7 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
     async def queryrow(self, sentence: str, *args):
         self._result = None
         error = None
+        started = self.start_timing()
         await self.valid_operation(sentence)
         try:
             stmt = await self._connection.prepare(sentence)
@@ -745,7 +832,7 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         except Exception as err: # pylint: disable=W0703
             error = f"Error on Query Row: {err}"
         finally:
-            self.generated_at()
+            self.generated_at(started)
             return await self._serializer(self._result, error) # pylint: disable=W0150
 
     async def execute(self, sentence: Any, *args, **kwargs) -> Optional[Any]:
@@ -967,7 +1054,6 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
                 format=file_type,
                 output=output,
             )
-            print(result)
             return result
         except UndefinedTableError as ex:
             raise StatementError(
@@ -1007,7 +1093,6 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
                 format=file_type,
                 source=source,
             )
-            print(result)
             return result
         except UndefinedTableError as ex:
             raise StatementError(
@@ -1149,7 +1234,11 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         insert a row from model.
         """
         try:
-            table = f"{_model.Meta.name}"
+            schema = ''
+            sc = _model.Meta.schema
+            if sc:
+                schema = f"{sc}."
+            table = f"{schema}{_model.Meta.name}"
         except AttributeError:
             table = _model.__name__
         cols = []
@@ -1192,7 +1281,7 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         try:
             columns = ",".join(cols)
             values = ",".join(["${}".format(a) for a in range(1, n)]) # pylint: disable=C0209
-            primary = "RETURNING *"
+            primary = "RETURNING {columns}"
             insert = f"INSERT INTO {table}({columns}) VALUES({values}) {primary}"
             self._logger.debug(f"INSERT: {insert}")
             stmt = await self._connection.prepare(insert)
@@ -1216,7 +1305,11 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         delete a row from model.
         """
         try:
-            table = f"{_model.Meta.name}"
+            schema = ''
+            sc = _model.Meta.schema
+            if sc:
+                schema = f"{sc}."
+            table = f"{schema}{_model.Meta.name}"
         except AttributeError:
             table = _model.__name__
         source = []
@@ -1255,7 +1348,11 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         Alternatives: Saving *dirty* status and previous value on dict
         """
         try:
-            table = f"{_model.Meta.name}"
+            schema = ''
+            sc = _model.Meta.schema
+            if sc:
+                schema = f"{sc}."
+            table = f"{schema}{_model.Meta.name}"
         except AttributeError:
             table = _model.__name__
         cols = []
@@ -1328,7 +1425,11 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         Returns one Row using Model.
         """
         try:
-            table = f"{_model.Meta.name}"
+            schema = ''
+            sc = _model.Meta.schema
+            if sc:
+                schema = f"{sc}."
+            table = f"{schema}{_model.Meta.name}"
         except AttributeError:
             table = _model.__name__
         fields = _model.columns()
@@ -1358,7 +1459,11 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         Filter a Model using Fields.
         """
         try:
-            table = f"{_model.Meta.name}"
+            schema = ''
+            sc = _model.Meta.schema
+            if sc:
+                schema = f"{sc}."
+            table = f"{schema}{_model.Meta.name}"
         except AttributeError:
             table = _model.__name__
         fields = _model.columns(_model)
@@ -1398,7 +1503,11 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
                 f'Missing Model for SELECT {kwargs!s}'
             ) from e
         try:
-            table = f"{model.Meta.name}"
+            schema = ''
+            sc = model.Meta.schema
+            if sc:
+                schema = f"{sc}."
+            table = f"{schema}{model.Meta.name}"
         except AttributeError:
             table = model.__name__
         if args:
@@ -1423,7 +1532,11 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         Get one row from model.
         """
         try:
-            table = f"{_model.Meta.name}"
+            schema = ''
+            sc = _model.Meta.schema
+            if sc:
+                schema = f"{sc}."
+            table = f"{schema}{_model.Meta.name}"
         except AttributeError:
             table = _model.__name__
         fields = _model.columns(_model)
@@ -1431,7 +1544,7 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         if args:
             columns = ','.join(args)
         else:
-            columns = '*'
+            columns = ','.join(fields) ## getting only selected fields
         for name, field in fields.items():
             if name in kwargs:
                 try:
@@ -1457,7 +1570,11 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         Get all rows on a Model.
         """
         try:
-            table = f"{_model.Meta.name}"
+            schema = ''
+            sc = _model.Meta.schema
+            if sc:
+                schema = f"{sc}."
+            table = f"{schema}{_model.Meta.name}"
         except AttributeError:
             table = _model.__name__
         if 'fields' in kwargs:
@@ -1478,7 +1595,11 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
         Deleting some records using Model.
         """
         try:
-            table = f"{_model.Meta.name}"
+            schema = ''
+            sc = _model.Meta.schema
+            if sc:
+                schema = f"{sc}."
+            table = f"{schema}{_model.Meta.name}"
         except AttributeError:
             table = _model.__name__
         fields = _model.columns(_model)
@@ -1512,11 +1633,11 @@ class pg(SQLDriver, DBCursorBackend, ModelBackend):
                 f'Missing Model for SELECT {kwargs!s}'
             ) from e
         try:
-            table = f"{model.Meta.name}"
-        except AttributeError:
-            table = model.__name__
-        try:
-            table = f"{model.Meta.name}"
+            schema = ''
+            sc = model.Meta.schema
+            if sc:
+                schema = f"{sc}."
+            table = f"{schema}{model.Meta.name}"
         except AttributeError:
             table = model.__name__
         fields = model.columns(model)
