@@ -2,31 +2,21 @@
 Basic, Abstract Model.
 """
 from __future__ import annotations
-import logging
+
 import inspect
+import logging
 import traceback
-from dataclasses import (
-    is_dataclass,
-    make_dataclass,
-    _MISSING_TYPE,
-    MISSING
-)
-from typing import (
-    Dict
-)
 from collections.abc import Awaitable
+from dataclasses import _MISSING_TYPE, MISSING, is_dataclass, make_dataclass
+
 from datamodel import BaseModel, Field
 from datamodel.base import Meta
-from datamodel.types import (
-    MODEL_TYPES
-)
+from datamodel.exceptions import ValidationError
+from datamodel.types import MODEL_TYPES
+
+from asyncdb.exceptions import ConnectionMissing, NoDataFound, ProviderError, StatementError
 from asyncdb.utils.modules import module_exists
-from asyncdb.exceptions import (
-    NoDataFound,
-    ProviderError,
-    StatementError,
-    ConnectionMissing
-)
+
 
 def is_missing(value):
     if value == _MISSING_TYPE:
@@ -231,6 +221,8 @@ class Model(BaseModel):
                 raise NoDataFound(
                     f"{self.Meta.name}: Data Not found"
                 )
+        except ValidationError:
+            raise
         except NoDataFound:
             raise
         except (AttributeError, StatementError) as err:
@@ -262,6 +254,8 @@ class Model(BaseModel):
             )
             if result:
                 return result
+        except ValidationError:
+            raise
         except (AttributeError, StatementError) as err:
             raise StatementError(
                 f"Error on Attribute {cls.Meta.name}: {err}"
@@ -344,6 +338,8 @@ class Model(BaseModel):
                 return [cls(**dict(r)) for r in result]
             else:
                 return []
+        except ValidationError:
+            raise
         except NoDataFound:
             raise
         except (AttributeError, StatementError) as err:
@@ -376,6 +372,8 @@ class Model(BaseModel):
                 return [cls(**dict(r)) for r in result]
             else:
                 return []
+        except ValidationError:
+            raise
         except NoDataFound:
             raise
         except (AttributeError, StatementError) as err:
@@ -404,11 +402,14 @@ class Model(BaseModel):
                 _model=cls, **kwargs
             )
             if result:
-                return cls(**dict(result))
+                fields = cls.get_fields(cls)
+                return cls(**{k:v for k,v in result.items() if k in fields})
             else:
                 raise NoDataFound(
                     message=f"Data not found over {cls.Meta.name!s}"
                 )
+        except ValidationError:
+            raise
         except NoDataFound as e:
             raise NoDataFound(
                 message=f"Data not found over {cls.Meta.name!s}"
@@ -439,6 +440,8 @@ class Model(BaseModel):
                 _model=cls, **kwargs
             )
             return [cls(**dict(row)) for row in result]
+        except ValidationError:
+            raise
         except StatementError:
             raise
         except ProviderError:
